@@ -38,9 +38,8 @@ struct SimpleScanPrivate
     GtkBuilder *builder;
 
     GtkWidget *window;
-    GtkWidget *scan_button_label, *continuous_scan_button_label, *page_label;
-    GtkWidget *device_combo, *mode_combo;
-    GtkTreeModel *device_model, *mode_model;
+    GtkWidget *mode_combo;
+    GtkTreeModel *mode_model;
     GtkWidget *preview_box, *preview_area, *preview_scroll;
     GtkWidget *page_delete_menuitem, *crop_rotate_menuitem;
 
@@ -49,13 +48,16 @@ struct SimpleScanPrivate
     GtkWidget *username_entry, *password_entry;
 
     GtkWidget *preferences_dialog;
-    GtkWidget *replace_pages_check;
+    GtkWidget *device_combo, *text_dpi_combo, *photo_dpi_combo;
+    GtkTreeModel *device_model, *text_dpi_model, *photo_dpi_model;
 
     Book *book;
     BookView *book_view;
     gboolean updating_page_menu;
-    Orientation default_orientation;
-    gboolean book_is_placeholder; // FIXME: Needs to be cleared when scan starts
+    gint default_page_width, default_page_height, default_page_dpi;
+    Orientation default_page_orientation;
+  
+    gboolean have_device_list;
 
     gchar *default_file_name;
     gboolean scanning;
@@ -131,40 +133,71 @@ ui_authorize (SimpleScan *ui, const gchar *resource, gchar **username, gchar **p
     *password = g_strdup (gtk_entry_get_text (GTK_ENTRY (ui->priv->password_entry)));
 }
 
-
 void
-ui_mark_devices_undetected (SimpleScan *ui)
+ui_set_scan_devices (SimpleScan *ui, GList *devices)
 {
     GtkTreeIter iter;
-    
-    if (gtk_tree_model_get_iter_first (ui->priv->device_model, &iter)) {
-        do {
-            gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 2, FALSE, -1);
-        } while (gtk_tree_model_iter_next (ui->priv->device_model, &iter));
+    GList *i;
+    gboolean have_iter;
+    gboolean have_selection;
+
+    if (!ui->priv->have_device_list) {
+        ui->priv->have_device_list = TRUE;
+
+        if (!devices) {
+            ui_show_error (ui,
+                           /* Warning displayed when no scanners are detected */
+                           _("No scanners detected"),
+                           /* Hint to user on why there are no scanners detected */
+                           _("Please check your scanner is connected and powered on"),
+                           FALSE);
+        }
+    }
+  
+    have_selection = gtk_combo_box_get_active (GTK_COMBO_BOX (ui->priv->device_combo)) >= 0;
+  
+    /* Remove disappeared devices */
+
+    do {
+        for (have_iter = gtk_tree_model_get_iter_first (ui->priv->device_model, &iter);
+             have_iter;
+             have_iter = gtk_tree_model_iter_next (ui->priv->device_model, &iter)) {
+            gchar *name;
+
+            gtk_tree_model_get (ui->priv->device_model, &iter, 0, &name, -1);
+            for (i = devices; i; i = i->next)
+                if (strcmp (name, ((ScanDevice *) i->data)->name) == 0)
+                    break;
+            g_free (name);
+
+            /* Device was removed */
+            if (i == NULL) {
+                gtk_list_store_remove (GTK_LIST_STORE (ui->priv->device_model), &iter);
+                break;
+            }
+        }
+    } while (have_iter);
+
+    /* Add new devices */
+    for (i = devices; i; i = i->next) {
+        ScanDevice *device = (ScanDevice *) i->data;
+
+        if (!find_scan_device (ui, device->name, &iter)) {
+            gtk_list_store_append (GTK_LIST_STORE (ui->priv->device_model), &iter);
+            gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device->name, 1, device->label, -1);
+
+            /* Select this device if none selected */
+            if (!have_selection) {
+                gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter);
+                have_selection = TRUE;
+            }
+        }
     }
 }
 
 
-void
-ui_add_scan_device (SimpleScan *ui, const gchar *device, const gchar *label)
-{
-    GtkTreeIter iter;
-    
-    if (!find_scan_device (ui, device, &iter)) {
-        gtk_list_store_append (GTK_LIST_STORE (ui->priv->device_model), &iter);
-        gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device, -1);
-    }
-
-    gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 1, label, 2, TRUE, -1);
-    
-    /* Select this device if none selected */
-    if (gtk_combo_box_get_active (GTK_COMBO_BOX (ui->priv->device_combo)) == -1)
-        gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter);
-}
-
-
-gchar *
-ui_get_selected_device (SimpleScan *ui)
+static gchar *
+get_selected_device (SimpleScan *ui)
 {
     GtkTreeIter iter;
 
@@ -199,12 +232,10 @@ add_default_page (SimpleScan *ui)
     if (book_get_n_pages (ui->priv->book) > 0)
         return;
 
-    /* Start with A4 white image at 72dpi */
-    // FIXME: Remember last page dimensions
-    book_append_page (ui->priv->book, 595, 842, 72, ui->priv->default_orientation);
-
-    /* Remove this page on the next scan */
-    ui->priv->book_is_placeholder = TRUE;
+    book_append_page (ui->priv->book,
+                      ui->priv->default_page_width, ui->priv->default_page_height,
+                      ui->priv->default_page_dpi,
+                      ui->priv->default_page_orientation);
 }
 
 
@@ -241,6 +272,32 @@ set_document_hint (SimpleScan *ui, const gchar *document_hint)
 }
 
 
+static gint
+get_text_dpi (SimpleScan *ui)
+{
+    GtkTreeIter iter;
+    gint dpi = 200;
+
+    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->text_dpi_combo), &iter))
+        gtk_tree_model_get (ui->priv->text_dpi_model, &iter, 0, &dpi, -1);
+    
+    return dpi;
+}
+
+
+static gint
+get_photo_dpi (SimpleScan *ui)
+{
+    GtkTreeIter iter;
+    gint dpi = 400;
+
+    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->photo_dpi_combo), &iter))
+        gtk_tree_model_get (ui->priv->photo_dpi_model, &iter, 0, &dpi, -1);
+    
+    return dpi;
+}
+
+
 static gchar *
 get_document_hint (SimpleScan *ui)
 {
@@ -254,28 +311,23 @@ get_document_hint (SimpleScan *ui)
 }
 
 
-static gboolean
-get_replace_pages (SimpleScan *ui)
-{
-    return ui->priv->book_is_placeholder || gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check));
-}
-
-
 void scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
     gchar *device, *mode;
+    gint dpi;
 
-    device = ui_get_selected_device (ui);
-    if (device) {
-        mode = get_document_hint (ui);
-        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
-                       FALSE, get_replace_pages (ui));
-        g_free (device);
-        g_free (mode);
-    }
+    device = get_selected_device (ui);
+    mode = get_document_hint (ui);
+    if (strcmp (mode, "text") == 0) 
+        dpi = get_text_dpi (ui);
+    else
+        dpi = get_photo_dpi (ui);
+    g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, dpi, mode, FALSE);
+    g_free (device);
+    g_free (mode);
 }
 
 
@@ -297,15 +349,17 @@ continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         g_signal_emit (G_OBJECT (ui), signals[STOP_SCAN], 0);
     } else {
         gchar *device, *mode;
+        gint dpi;
 
-        device = ui_get_selected_device (ui);
-        if (device) {
-            mode = get_document_hint (ui);
-            g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
-                           TRUE, get_replace_pages (ui));
-            g_free (device);
-            g_free (mode);
-        }
+        device = get_selected_device (ui);
+        mode = get_document_hint (ui);
+        if (strcmp (mode, "text") == 0) 
+            dpi = get_text_dpi (ui);
+        else
+            dpi = get_photo_dpi (ui);
+        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, dpi, mode, TRUE);
+        g_free (device);
+        g_free (mode);
     }
 }
 
@@ -390,7 +444,6 @@ rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         return;
     page = book_view_get_selected (ui->priv->book_view);
     page_rotate_left (page);
-    ui->priv->default_orientation = page_get_orientation (page);
 }
 
 
@@ -405,7 +458,7 @@ rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         return;
     page = book_view_get_selected (ui->priv->book_view);
     page_rotate_right (page);
-    ui->priv->default_orientation = page_get_orientation (page);
+    ui->priv->default_page_orientation = page_get_orientation (page);
 }
 
 
@@ -427,7 +480,21 @@ set_crop (SimpleScan *ui, const gchar *crop_name)
         page_set_no_crop (page);
         return;
     }
-    
+
+    if (strcmp (crop_name, "custom") == 0) {
+        gint width, height, crop_width, crop_height;
+
+        width = page_get_width (page);
+        height = page_get_height (page);
+
+        crop_width = (int) (width * 0.8 + 0.5);
+        crop_height = (int) (height * 0.8 + 0.5);
+        page_set_custom_crop (page, crop_width, crop_height);
+        page_move_crop (page, (width - crop_width) / 2, (height - crop_height) / 2);
+
+        return;
+    }
+
     page_set_named_crop (page, crop_name);
 }
 
@@ -451,6 +518,14 @@ custom_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
         set_crop (ui, "custom");
 }
 
+void crop_toolbutton_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+crop_toolbutton_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, "custom_crop_menuitem")), TRUE);
+}
+
 
 void four_by_six_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
@@ -537,6 +612,34 @@ page_delete_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
+static void
+on_file_type_changed (GtkTreeSelection *selection, GtkWidget *dialog)
+{
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    gchar *path, *filename, *extension, *new_filename;
+
+    if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+        return;
+
+    gtk_tree_model_get (model, &iter, 1, &extension, -1);
+    path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+    filename = g_path_get_basename (path);
+
+    /* Replace extension */
+    if (g_strrstr (filename, "."))
+        new_filename = g_strdup_printf ("%.*s%s", (int)(g_strrstr (filename, ".") - filename), filename, extension);
+    else
+        new_filename = g_strdup_printf ("%s%s", filename, extension);
+    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), new_filename);
+
+    g_free (path);
+    g_free (filename);
+    g_free (new_filename);
+    g_free (extension);
+}
+
+
 void save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
@@ -544,9 +647,30 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
     GtkWidget *dialog;
     gint response;
+    GtkFileFilter *filter;
+    GtkWidget *expander, *file_type_view;
+    GtkListStore *file_type_store;
+    GtkTreeIter iter;
+    GtkTreeViewColumn *column;
+    const gchar *extension;
+    gint i;
 
-    /* Title of save dialog */
-    dialog = gtk_file_chooser_dialog_new (_("Save As..."),
+    struct
+    {
+        gchar *label, *extension;
+    } file_types[] =
+    {
+        /* Save dialog: Label for saving in PDF format */
+        { _("PDF (multi-page document)"), ".pdf" },
+        /* Save dialog: Label for saving in JPEG format */
+        { _("JPEG (compressed)"), ".jpg" },
+        /* Save dialog: Label for saving in PNG format */
+        { _("PNG (lossless)"), ".png" },
+        { NULL, NULL }
+    };
+
+    dialog = gtk_file_chooser_dialog_new (/* Save dialog: Dialog title */
+                                          _("Save As..."),
                                           GTK_WINDOW (ui->priv->window),
                                           GTK_FILE_CHOOSER_ACTION_SAVE,
                                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -555,11 +679,65 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
     gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
     gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), ui->priv->default_file_name);
-    
+
+    /* Filter to only show images by default */
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name (filter,
+                              /* Save dialog: Filter name to show only image files */
+                              _("Image Files"));
+    gtk_file_filter_add_pixbuf_formats (filter);
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name (filter,
+                              /* Save dialog: Filter name to show all files */
+                              _("All Files"));
+    gtk_file_filter_add_pattern (filter, "*");
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+
+    expander = gtk_expander_new_with_mnemonic (/* */
+                                 _("Select File _Type"));
+    gtk_expander_set_spacing (GTK_EXPANDER (expander), 5);
+    gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (dialog), expander);
+  
+    extension = strstr (ui->priv->default_file_name, ".");
+    if (!extension)
+        extension = "";
+
+    file_type_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+    for (i = 0; file_types[i].label; i++) {
+        gtk_list_store_append (file_type_store, &iter);
+        gtk_list_store_set (file_type_store, &iter, 0, file_types[i].label, 1, file_types[i].extension, -1);
+    }
+
+    file_type_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (file_type_store));
+    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (file_type_view), FALSE);
+    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (file_type_view), TRUE);
+    column = gtk_tree_view_column_new_with_attributes ("",
+                                                       gtk_cell_renderer_text_new (),
+                                                       "text", 0, NULL);
+    gtk_tree_view_append_column (GTK_TREE_VIEW (file_type_view), column);
+    gtk_container_add (GTK_CONTAINER (expander), file_type_view);
+
+    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (file_type_store), &iter)) {
+        do {
+            gchar *e;
+            gtk_tree_model_get (GTK_TREE_MODEL (file_type_store), &iter, 1, &e, -1);
+            if (strcmp (extension, e) == 0)
+                gtk_tree_selection_select_iter (gtk_tree_view_get_selection (GTK_TREE_VIEW (file_type_view)), &iter);
+            g_free (e);
+        } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (file_type_store), &iter));
+    }
+    g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (file_type_view)),
+                      "changed",
+                      G_CALLBACK (on_file_type_changed),
+                      dialog);
+
+    gtk_widget_show_all (expander);
+
     response = gtk_dialog_run (GTK_DIALOG (dialog));
     if (response == GTK_RESPONSE_ACCEPT) {
         gchar *uri;
-        
+
         uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
         g_signal_emit (G_OBJECT (ui), signals[SAVE], 0, uri);
 
@@ -633,118 +811,33 @@ print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
-static void
-load_device_cache (SimpleScan *ui)
+void help_contents_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+help_contents_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    gchar *filename;
-    GKeyFile *key_file;
-    gboolean result;
+    GdkScreen *screen;
     GError *error = NULL;
-    
-    filename = g_build_filename (g_get_user_cache_dir (), "simple-scan", "device_cache", NULL);
-    
-    key_file = g_key_file_new ();
-    result = g_key_file_load_from_file (key_file, filename, G_KEY_FILE_NONE, &error);
-    if (error) {
-        if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
-            g_warning ("Error loading device cache file: %s", error->message);
+
+    screen = gtk_widget_get_screen (GTK_WIDGET (ui->priv->window));
+    gtk_show_uri (screen, "ghelp:simple-scan", gtk_get_current_event_time (), &error);
+
+    if (error != NULL)
+    {
+        GtkWidget *d;
+        /* Error message displayed when unable to launch help browser */
+        const char *message = _("Unable to open help file");
+
+        d = gtk_message_dialog_new (GTK_WINDOW (ui->priv->window),
+                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                    GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                                    "%s", message);
+        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (d),
+                                                  "%s", error->message);
+        g_signal_connect (d, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+        gtk_window_present (GTK_WINDOW (d));
+
         g_error_free (error);
-        error = NULL;
-    }
-    if (result) {
-        gchar **groups, **group_iter;
-
-        groups = g_key_file_get_groups (key_file, NULL);
-        for (group_iter = groups; *group_iter; group_iter++) {
-            gchar *label, *device;
-
-            label = *group_iter;
-            device = g_key_file_get_value (key_file, label, "device", &error);
-            if (error) {
-                g_warning ("Error getting device name for label '%s': %s", label, error->message);
-                g_error_free (error);
-                error = NULL;
-            }
-            
-            if (device)
-                ui_add_scan_device (ui, device, label);
-
-            g_free (device);
-        }
-
-        g_strfreev (groups);
-    }
-
-    g_free (filename);
-    g_key_file_free (key_file);
-}
-
-
-static void
-save_device_cache (SimpleScan *ui)
-{
-    GtkTreeModel *model;
-    GtkTreeIter iter;
-
-    g_debug ("Saving device cache");
-
-    model = ui->priv->device_model;
-    if (gtk_tree_model_get_iter_first (model, &iter)) {
-        GKeyFile *key_file;
-        gchar *data;
-        gsize data_length;
-        GError *error = NULL;
-
-        key_file = g_key_file_new ();
-        do {
-            gchar *name, *label;
-            gboolean detected;
-            
-            gtk_tree_model_get (model, &iter, 0, &name, 1, &label, 2, &detected, -1);
-            
-            if (detected) {
-                g_debug ("Storing device '%s' in cache", name);
-                g_key_file_set_value (key_file, label, "device", name);
-            }
-
-            g_free (name);
-            g_free (label);
-        } while (gtk_tree_model_iter_next (model, &iter));
-        
-        data = g_key_file_to_data (key_file, &data_length, &error);
-        if (data) {
-            gchar *dir, *filename;
-            GFile *file;
-            GFileOutputStream *stream;
-            GError *error = NULL;
-
-            dir = g_build_filename (g_get_user_cache_dir (), "simple-scan", NULL);
-            g_mkdir_with_parents (dir, 0700);
-            filename = g_build_filename (dir, "device_cache", NULL);
-
-            file = g_file_new_for_path (filename);
-            stream = g_file_replace (file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error);
-            if (error) {
-                g_warning ("Error writing device cache: %s", error->message);
-                g_error_free (error);
-                error = NULL;
-            }
-            if (stream) {
-                g_output_stream_write_all (G_OUTPUT_STREAM (stream), data, data_length, NULL, NULL, &error);
-                if (error) {
-                    g_warning ("Error writing device cache: %s", error->message);
-                    g_error_free (error);
-                    error = NULL;
-                }
-                g_output_stream_close (G_OUTPUT_STREAM (stream), NULL, NULL);
-            }
-            g_free (data);
-
-            g_free (filename);
-            g_free (dir);        
-        }
-
-        g_key_file_free (key_file);
     }
 }
 
@@ -797,9 +890,7 @@ quit (SimpleScan *ui)
     char *device, *document_type;
     gint i;
 
-    save_device_cache (ui);
-
-    device = ui_get_selected_device (ui);
+    device = get_selected_device (ui);
     if (device) {
         gconf_client_set_string(ui->priv->client, "/apps/simple-scan/selected_device", device, NULL);
         g_free (device);
@@ -808,17 +899,19 @@ quit (SimpleScan *ui)
     document_type = get_document_hint (ui);
     gconf_client_set_string(ui->priv->client, "/apps/simple-scan/document_type", document_type, NULL);
     g_free (document_type);
-
-    gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/replace_pages",
-                          gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check)), NULL);
+    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/text_dpi", get_text_dpi (ui), NULL);
+    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/photo_dpi", get_photo_dpi (ui), NULL);
 
     gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_width", ui->priv->window_width, NULL);
     gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_height", ui->priv->window_height, NULL);
     gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/window_is_maximized", ui->priv->window_is_maximized, NULL);
 
-    for (i = 0; orientation_keys[i].key != NULL && orientation_keys[i].orientation != ui->priv->default_orientation; i++);
+    for (i = 0; orientation_keys[i].key != NULL && orientation_keys[i].orientation != ui->priv->default_page_orientation; i++);
     if (orientation_keys[i].key != NULL)
         gconf_client_set_string(ui->priv->client, "/apps/simple-scan/scan_direction", orientation_keys[i].key, NULL);
+    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_width", ui->priv->default_page_width, NULL);
+    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_height", ui->priv->default_page_height, NULL);
+    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_dpi", ui->priv->default_page_dpi, NULL);
    
     g_signal_emit (G_OBJECT (ui), signals[QUIT], 0);
 }
@@ -868,6 +961,27 @@ window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
 }
 
 
+static void
+page_changed_cb (Page *page, SimpleScan *ui)
+{
+    ui->priv->default_page_width = page_get_scan_width (page);
+    ui->priv->default_page_height = page_get_scan_height (page);
+    ui->priv->default_page_dpi = page_get_dpi (page);
+    ui->priv->default_page_orientation = page_get_orientation (page);
+}
+
+
+static void
+page_added_cb (Book *book, Page *page, SimpleScan *ui)
+{
+    ui->priv->default_page_width = page_get_scan_width (page);
+    ui->priv->default_page_height = page_get_scan_height (page);
+    ui->priv->default_page_dpi = page_get_dpi (page);
+    ui->priv->default_page_orientation = page_get_orientation (page);
+    g_signal_connect (page, "image-changed", G_CALLBACK (page_changed_cb), ui);
+}
+
+
 static void
 page_removed_cb (Book *book, Page *page, SimpleScan *ui)
 {
@@ -877,10 +991,58 @@ page_removed_cb (Book *book, Page *page, SimpleScan *ui)
 
 
 static void
-book_cleared_cb (Book *book, SimpleScan *ui)
+set_dpi_combo (GtkWidget *combo, gint default_dpi, gint current_dpi)
 {
-    /* Must have been cleared for next scan */
-    ui->priv->book_is_placeholder = FALSE;
+    struct
+    {
+       gint dpi;
+       const gchar *label;
+    } scan_resolutions[] =
+    {
+      /* Preferences dialog: Label for minimum resolution in resolution list */
+      { 75,  _("%d dpi (draft)") },
+      /* Preferences dialog: Label for resolution value in resolution list (dpi = dots per inch) */
+      { 150, _("%d dpi") },
+      { 200, _("%d dpi") },
+      { 400, _("%d dpi") },
+      { 600, _("%d dpi") },
+      { 800, _("%d dpi") },
+      { 1000, _("%d dpi") },
+      /* Preferences dialog: Label for maximum resolution in resolution list */      
+      { 1200, _("%d dpi (high resolution)") },
+      { -1, NULL }
+    };
+    GtkCellRenderer *renderer;
+    GtkTreeModel *model;
+    gint i;
+
+    renderer = gtk_cell_renderer_text_new();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
+    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo), renderer, "text", 1);
+
+    model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+    for (i = 0; scan_resolutions[i].dpi > 0; i++)
+    {
+        GtkTreeIter iter;
+        gchar *label;
+        gint dpi;
+
+        dpi = scan_resolutions[i].dpi;
+
+        if (dpi == default_dpi)
+            label = g_strdup_printf (/* Preferences dialog: Label for default resolution in resolution list */
+                                     _("%d dpi (default)"), dpi);
+        else
+            label = g_strdup_printf (scan_resolutions[i].label, dpi);
+
+        gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+        gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, dpi, 1, label, -1);
+
+        if (dpi == current_dpi)
+            gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter);
+
+        g_free (label);
+    }
 }
 
 
@@ -891,7 +1053,7 @@ ui_load (SimpleScan *ui)
     GError *error = NULL;
     GtkCellRenderer *renderer;
     gchar *device, *document_type, *scan_direction;
-    gboolean replace_pages;
+    gint dpi;
 
     builder = ui->priv->builder = gtk_builder_new ();
     gtk_builder_add_from_file (builder, UI_DIR "simple-scan.ui", &error);
@@ -908,11 +1070,6 @@ ui_load (SimpleScan *ui)
     gtk_builder_connect_signals (builder, ui);
 
     ui->priv->window = GTK_WIDGET (gtk_builder_get_object (builder, "simple_scan_window"));
-    ui->priv->scan_button_label = GTK_WIDGET (gtk_builder_get_object (builder, "scan_button_label"));
-    ui->priv->continuous_scan_button_label = GTK_WIDGET (gtk_builder_get_object (builder, "continuous_scan_button_label"));
-    ui->priv->page_label = GTK_WIDGET (gtk_builder_get_object (builder, "page_label"));
-    ui->priv->device_combo = GTK_WIDGET (gtk_builder_get_object (builder, "device_combo"));
-    ui->priv->device_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->device_combo));
     ui->priv->mode_combo = GTK_WIDGET (gtk_builder_get_object (builder, "mode_combo"));
     ui->priv->mode_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->mode_combo));
     ui->priv->preview_box = GTK_WIDGET (gtk_builder_get_object (builder, "preview_vbox"));
@@ -921,25 +1078,37 @@ ui_load (SimpleScan *ui)
     ui->priv->page_delete_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "page_delete_menuitem"));
     ui->priv->crop_rotate_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "crop_rotate_menuitem"));
 
+    renderer = gtk_cell_renderer_text_new();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, TRUE);
+    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, "text", 1);
+    gtk_combo_box_set_active (GTK_COMBO_BOX (ui->priv->mode_combo), 0);
+
     ui->priv->authorize_dialog = GTK_WIDGET (gtk_builder_get_object (builder, "authorize_dialog"));
     ui->priv->authorize_label = GTK_WIDGET (gtk_builder_get_object (builder, "authorize_label"));
     ui->priv->username_entry = GTK_WIDGET (gtk_builder_get_object (builder, "username_entry"));
     ui->priv->password_entry = GTK_WIDGET (gtk_builder_get_object (builder, "password_entry"));
    
     ui->priv->preferences_dialog = GTK_WIDGET (gtk_builder_get_object (builder, "preferences_dialog"));
-    ui->priv->replace_pages_check = GTK_WIDGET (gtk_builder_get_object (builder, "replace_pages_check"));
+    ui->priv->device_combo = GTK_WIDGET (gtk_builder_get_object (builder, "device_combo"));
+    ui->priv->device_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->device_combo));
+    ui->priv->text_dpi_combo = GTK_WIDGET (gtk_builder_get_object (builder, "text_dpi_combo"));
+    ui->priv->text_dpi_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->text_dpi_combo));
+    ui->priv->photo_dpi_combo = GTK_WIDGET (gtk_builder_get_object (builder, "photo_dpi_combo"));
+    ui->priv->photo_dpi_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->photo_dpi_combo));
+
+    dpi = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/text_dpi", NULL);
+    if (dpi <= 0)
+        dpi = 200;
+    set_dpi_combo (ui->priv->text_dpi_combo, 200, dpi);
+    dpi = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/photo_dpi", NULL);
+    if (dpi <= 0)
+        dpi = 400;
+    set_dpi_combo (ui->priv->photo_dpi_combo, 400, dpi);
 
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, TRUE);
     gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, "text", 1);
 
-    renderer = gtk_cell_renderer_text_new();
-    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, TRUE);
-    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, "text", 1);
-    gtk_combo_box_set_active (GTK_COMBO_BOX (ui->priv->mode_combo), 0);
-
-    /* Load previously detected scanners and select the last used one */
-    load_device_cache (ui);
     device = gconf_client_get_string (ui->priv->client, "/apps/simple-scan/selected_device", NULL);
     if (device) {
         GtkTreeIter iter;
@@ -954,9 +1123,6 @@ ui_load (SimpleScan *ui)
         g_free (document_type);
     }
 
-    replace_pages = gconf_client_get_bool (ui->priv->client, "/apps/simple-scan/replace_pages", NULL);
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check), replace_pages);
-    
     ui->priv->book_view = book_view_new ();
     g_signal_connect (ui->priv->book_view, "page-selected", G_CALLBACK (page_selected_cb), ui);
     book_view_set_widgets (ui->priv->book_view,
@@ -965,14 +1131,24 @@ ui_load (SimpleScan *ui)
                            ui->priv->preview_scroll,
                            GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
 
-    /* Find default scan direction */
+    /* Find default page details */
     scan_direction = gconf_client_get_string(ui->priv->client, "/apps/simple-scan/scan_direction", NULL);
+    ui->priv->default_page_orientation = TOP_TO_BOTTOM;
     if (scan_direction) {
         gint i;
         for (i = 0; orientation_keys[i].key != NULL && strcmp (orientation_keys[i].key, scan_direction) != 0; i++);
         if (orientation_keys[i].key != NULL)
-            ui->priv->default_orientation = orientation_keys[i].orientation;
+            ui->priv->default_page_orientation = orientation_keys[i].orientation;
     }
+    ui->priv->default_page_width = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_width", NULL);
+    if (ui->priv->default_page_width <= 0)
+        ui->priv->default_page_width = 595;
+    ui->priv->default_page_height = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_height", NULL);
+    if (ui->priv->default_page_height <= 0)
+        ui->priv->default_page_height = 842;
+    ui->priv->default_page_dpi = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_dpi", NULL);
+    if (ui->priv->default_page_dpi <= 0)
+        ui->priv->default_page_dpi = 72;
 
     /* Restore window size */
     ui->priv->window_width = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/window_width", NULL);
@@ -1074,20 +1250,20 @@ ui_start (SimpleScan *ui)
 
 /* Generated with glib-genmarshal */
 static void
-g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
-                                                             GValue       *return_value G_GNUC_UNUSED,
-                                                             guint         n_param_values,
-                                                             const GValue *param_values,
-                                                             gpointer      invocation_hint G_GNUC_UNUSED,
-                                                             gpointer      marshal_data)
+g_cclosure_user_marshal_VOID__STRING_INT_STRING_BOOLEAN (GClosure     *closure,
+                                                         GValue       *return_value G_GNUC_UNUSED,
+                                                         guint         n_param_values,
+                                                         const GValue *param_values,
+                                                         gpointer      invocation_hint G_GNUC_UNUSED,
+                                                         gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_STRING_BOOLEAN_BOOLEAN) (gpointer       data1,
-                                                                    gconstpointer  arg_1,
-                                                                    gconstpointer  arg_2,
-                                                                    gboolean       arg_3,
-                                                                    gboolean       arg_4,
-                                                                    gpointer       data2);
-  register GMarshalFunc_VOID__STRING_STRING_BOOLEAN_BOOLEAN callback;
+  typedef void (*GMarshalFunc_VOID__STRING_INT_STRING_BOOLEAN) (gpointer       data1,
+                                                                gconstpointer  arg_1,
+                                                                gint           arg_2,
+                                                                gconstpointer  arg_3,
+                                                                gboolean       arg_4,
+                                                                gpointer       data2);
+  register GMarshalFunc_VOID__STRING_INT_STRING_BOOLEAN callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -1103,12 +1279,12 @@ g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN (GClosure     *closu
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__STRING_STRING_BOOLEAN_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__STRING_INT_STRING_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_value_get_string (param_values + 1),
-            g_value_get_string (param_values + 2),
-            g_value_get_boolean (param_values + 3),
+            g_value_get_int (param_values + 2),
+            g_value_get_string (param_values + 3),
             g_value_get_boolean (param_values + 4),
             data2);
 }
@@ -1123,8 +1299,8 @@ ui_class_init (SimpleScanClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SimpleScanClass, start_scan),
                       NULL, NULL,
-                      g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN,
-                      G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
+                      g_cclosure_user_marshal_VOID__STRING_INT_STRING_BOOLEAN,
+                      G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_BOOLEAN);
     signals[STOP_SCAN] =
         g_signal_new ("stop-scan",
                       G_TYPE_FROM_CLASS (klass),
@@ -1169,7 +1345,7 @@ ui_init (SimpleScan *ui)
 
     ui->priv->book = book_new ();
     g_signal_connect (ui->priv->book, "page-removed", G_CALLBACK (page_removed_cb), ui);
-    g_signal_connect (ui->priv->book, "cleared", G_CALLBACK (book_cleared_cb), ui);
+    g_signal_connect (ui->priv->book, "page-added", G_CALLBACK (page_added_cb), ui);
    
     ui->priv->client = gconf_client_get_default();
     gconf_client_add_dir(ui->priv->client, "/apps/simple-scan", GCONF_CLIENT_PRELOAD_NONE, NULL);
